package com.accenture.eVote.elettorims.service;

import com.accenture.eVote.elettorims.dto.ElettoreDTO;
import com.accenture.eVote.elettorims.dto.ElettoreSchedaDTO;
import com.accenture.eVote.elettorims.entity.Elettore;
import com.accenture.eVote.elettorims.entity.ElettoreScheda;
import com.accenture.eVote.elettorims.entity.ListaAttributiQualificati;
import com.accenture.eVote.elettorims.entity.RicevutaScheda;
import com.accenture.eVote.elettorims.entity.id.IdElettoreScheda;
import com.accenture.eVote.elettorims.mapper.ElettoreMapper;
import com.accenture.eVote.elettorims.mapper.ElettoreSchedaMapper;
import com.accenture.eVote.elettorims.model.request.OracleValidateRequest;
import com.accenture.eVote.elettorims.model.response.OracleValidateResponse;
import com.accenture.eVote.elettorims.model.response.SchedeAttiveResponse;
import com.accenture.eVote.elettorims.model.response.VerificaVotoResponse;
import com.accenture.eVote.elettorims.repository.ElettoreRepository;
import com.accenture.eVote.elettorims.repository.ElettoreSchedaRepository;
import com.accenture.eVote.elettorims.repository.ListaAttributiQualificatiRepository;
import com.accenture.eVote.elettorims.repository.RicevutaSchedaRepository;
import com.accenture.eVote.elettorims.util.Crittografia;
import com.accenture.eVote.elettorims.util.enums.ElettoriEnum;
import com.accenture.eVote.elettorims.util.enums.RicevutaSchedaEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.web.reactive.function.client.WebClient;
import org.springframework.web.reactive.function.client.WebClientResponseException;
import reactor.core.publisher.Mono;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@Service
public class ElettoreService {
    private final WebClient.Builder webClientBuilder;

    public ElettoreService(WebClient.Builder webClientBuilder) {
        this.webClientBuilder = webClientBuilder;
    }

    @Autowired
    private ElettoreRepository elettoreRepository;
    @Autowired
    private ElettoreSchedaRepository elettoreSchedaRepository;
    @Autowired
    private ListaAttributiQualificatiRepository listaAttributiQualificatiRepository;
    @Autowired
    private RicevutaSchedaRepository ricevutaSchedaRepository;
    @Autowired
    private ElettoreMapper elettoreMapper;
    @Autowired
    private ElettoreSchedaMapper elettoreSchedaMapper;
    @Autowired
    private Crittografia crittografia;
    @Value("${formati.request.data}")
    private String formatoData;
    @Value("${url.oracle.qa.validate}")
    private String urlOracleValidate;
    @Value("${url.schede.valide}")
    private String urlSchedeValide;

    public ElettoreDTO getElettoreById(Long id) {
        return crittografia.decodificaDatiSensibili(elettoreMapper.toDto(elettoreRepository.findById(id).get()));
    }

    public ElettoreDTO getElettoreByCf(String cf) {
        //codifica codice fiscale
        cf = crittografia.codificaCf(cf);
        return crittografia.decodificaDatiSensibili(elettoreMapper.toDto(elettoreRepository.findByCodiceFiscale(cf)));
    }

    //da cambiare return; oltre a elettore aggiungere anche data elezione
    public ElettoreDTO getRicevuteVoto(Long userId) {
        Optional<Elettore> elettoreOptional = elettoreRepository.findById(userId);
        if (elettoreOptional.isPresent()) {
            Elettore elettore = elettoreOptional.get();
            try {
                ElettoreDTO elettoreDTO = crittografia.decodificaDatiSensibili(elettoreMapper.toDto(elettore));
                elettoreDTO.setDataNascita(elettoreRepository.findById(elettoreDTO.getUserId()).get().getDataNascita());
                return elettoreDTO;

            } catch (Exception e) {
                return null;
            }

        } else {
            return null;
        }

    }


    public List<ElettoreSchedaDTO> setAccettazioneSchede(List<ElettoreSchedaDTO> listElettoreSchedaDTO, Long userId) {

        List<ElettoreSchedaDTO> listSchedeResult = new ArrayList<>();

        listElettoreSchedaDTO.forEach(scheda -> {
            IdElettoreScheda idElettoreScheda = new IdElettoreScheda();
            idElettoreScheda.setUserId(userId);
            idElettoreScheda.setSchedaVoto(scheda.getSchedaVoto());
            Optional<ElettoreScheda> existingRecord = elettoreSchedaRepository.findById(idElettoreScheda);

            if (!existingRecord.isPresent()) {
                ElettoreScheda elettoreScheda = elettoreSchedaMapper.toEntity(scheda);
                elettoreScheda.setUserId(userId);
                elettoreSchedaRepository.save(elettoreScheda);
                listSchedeResult.add(scheda);
            }
        });

        return listSchedeResult;
    }

    public VerificaVotoResponse verificaVoto(Long userId) throws InterruptedException{

        // recupero se presente l'attributo qualificato
        ListaAttributiQualificati attributiQualificati = listaAttributiQualificatiRepository.findByUserId(userId);

        if (attributiQualificati != null && attributiQualificati.getAttributoQualificato() != null) {

            // verifico che l'attributo preso sia stato speso correttamente

            // Genero la richiesta di validazione
            OracleValidateRequest oracleValidateRequest = new OracleValidateRequest(attributiQualificati.getAttributoQualificato());
            try {                

                // Chiedo verifica dell'attributo qualificato
                OracleValidateResponse oracleValidateResponse = webClientBuilder.build()
                        .post()
                        .uri(urlOracleValidate)
                        .bodyValue(oracleValidateRequest)
                        .retrieve()
                        // Verifico che lo stato della risposta non sia 422 in quel caso rilancio un eccezione
                        .onStatus(
                                status -> status.equals(HttpStatus.UNPROCESSABLE_ENTITY),
                                response -> Mono.error(new WebClientResponseException(HttpStatus.UNPROCESSABLE_ENTITY.value(), "Attributo qualificato speso", response.headers().asHttpHeaders(), null, null))
                        )
                        .bodyToMono(OracleValidateResponse.class)
                        .block();

                // In qualcunque caso diverso da 422 l'elettore avrà ancora diritto al voto
                return new VerificaVotoResponse(false);
            } catch (WebClientResponseException e) {
                if (e.getStatusCode().equals(HttpStatus.UNPROCESSABLE_ENTITY)) {
                    return new VerificaVotoResponse(true); // l'utente ha votato
                } else {
                    return new VerificaVotoResponse(false); // l'utente non ha votato
                }
            } catch (Exception e) {
                // Se in generale abbiamo un errore restituiamo l'errore
                e.printStackTrace();
                throw e;
            }

        }
        // Nel caso in cui non sia presente l'attributo di default non ha votato
        return new VerificaVotoResponse(false);
    }

    public Boolean checkEmail(ElettoreDTO elettore) {
        return elettore.getEmailPec() != null && !elettore.getEmailPec().isEmpty();
    }

    public boolean verificaSchede(List<ElettoreSchedaDTO> listSchede) {

        // Recupero da schede-ms la lista di schede attive per l'elezione corrente
        SchedeAttiveResponse schedeAttiveResponse = webClientBuilder.build()
                .get()
                .uri(urlSchedeValide)
                .retrieve()
                .bodyToMono(SchedeAttiveResponse.class)
                .block();

        // Confronto le schede inviate con quelle attive

        return schedeAttiveResponse.getIdSchedeAttive().containsAll(listSchede.stream()
                .map(ElettoreSchedaDTO::getSchedaVoto)
                .collect(Collectors.toList()));
    }


    public void verificaRicevutaScheda(Long userId) {
        // Verifico che l'utente sia all'interno della tabella di ricevuta voto
        if (ricevutaSchedaRepository.findByUserId(userId).isEmpty()) {
            List<Long> idSchedeAccettate = elettoreSchedaRepository.findIdsSchedeAccettate(userId, ElettoriEnum.SCHEDA_ACCETTATA.getValore());
            if (!idSchedeAccettate.isEmpty()) {
                idSchedeAccettate.forEach(scheda -> ricevutaSchedaRepository.save(new RicevutaScheda(userId, scheda, RicevutaSchedaEnum.VOTATO.getFlag())));
            }
        }
    }
}
