package com.accenture.eVote.votoms.service;

import com.accenture.eVote.votoms.dto.EvoteRequestType;
import com.accenture.eVote.votoms.dto.RequestVotoDTO;
import com.accenture.eVote.votoms.entity.ConfigParametri;
import com.accenture.eVote.votoms.entity.Territorio;
import com.accenture.eVote.votoms.entity.id.IdConfigParametri;
import com.accenture.eVote.votoms.model.ElezioneResponse;
import com.accenture.eVote.votoms.model.SchedaVerificaIdResponse;
import com.accenture.eVote.votoms.repository.ConfigParametriRepository;
import com.accenture.eVote.votoms.repository.TerritorioRepository;
import com.accenture.eVote.votoms.util.Crittografia;
import com.accenture.eVote.votoms.util.ValidatoreVoto;
import com.accenture.eVote.votoms.util.enums.ParametriEnum;
import com.accenture.eVote.votoms.util.exception.IncongruenzaSchedeCandidatiException;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.reactive.function.client.WebClient;

import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;

@Service
public class VotoService {

    private final WebClient.Builder webClientBuilder;

    public VotoService(WebClient.Builder webClientBuilder) {
        this.webClientBuilder = webClientBuilder;
    }
    private final Logger log = LoggerFactory.getLogger(VotoService.class);

    @Autowired
    private ConfigParametriRepository configParametriRepository;
    @Autowired
    private TerritorioRepository territorioRepository;
    @Autowired
    private Crittografia crittografia;
    @Autowired
    private ValidatoreVoto validatoreVoto;
    @Value("${url.schede.elezioni.base}")
    private String baseSchede;
    @Value("${url.schede.elezioni.endpoint.elezione}")
    private String endpointSchedeElezione;
    @Value("${url.schede.elezioni.endpoint.verificaid}")
    private String endpointSchedeVerifica;
    @Value("${sicurezza.decodifica.chiave.voto}")
    private String chiaveDecriptVoto;

    public Long getTimer(Long tipologiaElezione) {
        IdConfigParametri idConfigParametri = new IdConfigParametri(ParametriEnum.TIMER.getValore(), tipologiaElezione);
        Optional<ConfigParametri> configParametri = configParametriRepository.findById(idConfigParametri);
        return configParametri.map(ConfigParametri::getValoreParametro).orElse(null);
    }

    public byte[] setVoto(String requestVotoCritatta) throws Exception {

        String requestDecritatta = crittografia.decodificaStringa(requestVotoCritatta, chiaveDecriptVoto);
        ObjectMapper objectMapper = new ObjectMapper();

        try {
            // Converto la stringa decritatta in RequestVotoDTO
            RequestVotoDTO requestVotoDTO = objectMapper.readValue(requestDecritatta, new TypeReference<RequestVotoDTO>(){});

            // Recupero gli id consensiti da schede
            List<SchedaVerificaIdResponse> schedaVerificaIdResponseList = requestVotoDTO.getListSchedeVoto().stream()
                    .map(schedeVotoDTO -> {
                        return webClientBuilder
                                .baseUrl(baseSchede)
                                .build()
                                .get()
                                .uri(uriBuilder -> uriBuilder.path(endpointSchedeVerifica)
                                        .queryParam("idScheda", schedeVotoDTO.getChiaveScheda())
                                        .build())
                                .retrieve()
                                .bodyToMono(SchedaVerificaIdResponse.class)
                                .block();

                    }).collect(Collectors.toList());

            // Validazione id ricevuti e numero preferenze
            if(!validatoreVoto.validazioneVoto(requestVotoDTO, schedaVerificaIdResponseList)){
                throw new IncongruenzaSchedeCandidatiException("Liste o candidati incongruenti con le schede proposte");
            };

            // Utilizziamo in questo caso una chiamata sincrona

            List<ElezioneResponse> elezioneResponseList = webClientBuilder
                    .baseUrl(baseSchede)
                    .build()
                    .get()
                    .uri(uriBuilder -> uriBuilder.path(endpointSchedeElezione)
                            .queryParam("idSchede", requestVotoDTO.getListSchedeVoto().stream()
                                    .map(schedaVoto -> schedaVoto.getChiaveScheda().toString())
                                    .collect(Collectors.joining(",")))
                            .build())
                    .retrieve()
                    .bodyToFlux(ElezioneResponse.class)
                    .collectList()
                    .block();

            // Mapping dell'id elezione nella requestVotoDTO
            requestVotoDTO.setListSchedeVoto(requestVotoDTO.getListSchedeVoto().stream()
                    .map(scheda -> {
                        scheda.getLista().setIdElezione(
                                // Recupero l'id elezione per match di idScheda
                                Objects.requireNonNull(elezioneResponseList.stream()
                                                .filter(elezioneResponse -> elezioneResponse.getIdScheda().equals(scheda.getChiaveScheda()))
                                                .findFirst().orElse(null))
                                        .getIdElezione());
                        return scheda;
                    })
                    .collect(Collectors.toList()));

            // Mapping dei dati territorio
            requestVotoDTO.setListSchedeVoto(requestVotoDTO.getListSchedeVoto().stream()
                    .map(scheda -> {
                        // Recupero e setting del territorio padre per id Elezione e Geo Id
                        Territorio territorioPadre = territorioRepository.findByGeoIdAndIdElezione(requestVotoDTO.getGeoId(), scheda.getLista().getIdElezione());
                        scheda.getLista().setTerritorioPadre(territorioPadre.getId());
                        scheda.getLista().setTerritorioPadreDesc(territorioPadre.getDescrizioneEnte());

                        // Recupero e setting delle informazioni del territorio figlio
                        Territorio territorioFiglio = territorioRepository.findByIdTerritorioPadre(territorioPadre.getId());
                        scheda.getLista().setTerritorio(territorioFiglio.getId());
                        scheda.getLista().setTerritorioDesc(territorioFiglio.getDescrizioneEnte());
                        return scheda;
                    }).collect(Collectors.toList()));

            // Maping su json
            String listVotiCrittata = objectMapper.writeValueAsString(crittografia.codificaVoto(objectMapper.writeValueAsString(requestVotoDTO.getListSchedeVoto())));
            EvoteRequestType reqOracle = null;

            String requestCrittata = objectMapper.writeValueAsString(reqOracle);
            return crittografia.codificaOracleRequest(requestCrittata);
        } catch (JsonProcessingException e) {
            e.printStackTrace();
            throw new RuntimeException(e);
        }
    }

}
