package it.evota.validate.controller;

import java.util.Date;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnExpression;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import it.evota.validate.dto.CountAQResultType;
import it.evota.validate.dto.ErrorResponseType;
import it.evota.validate.dto.VerifyAQRequestType;
import it.evota.validate.dto.VerifyAQResponseType;
import it.evota.validate.service.BcValidateService;
import it.evota.validate.util.exception.EVoteAppException;

@RestController
@ConditionalOnExpression("${verify.enabled:false}")
public class VerifyQualifyAttributeController {

	@Autowired
	BcValidateService service;

	Logger logger = LogManager.getLogger(getClass());

	@Operation(summary = "Verify a previous generation")
	@ApiResponses(value = { @ApiResponse(responseCode = "200", description = "Ok", content = {
			@Content(mediaType = "application/json", schema = @Schema(implementation = VerifyAQResponseType.class)) }),
			@ApiResponse(responseCode = "422", description = "Qualified attribute not found:VERIFY_ERROR_001 , Qualified attribute already verified:VERIFY_ERROR_002 ", content = {
					@Content(mediaType = "application/problem+json", schema = @Schema(implementation = ErrorResponseType.class)) }),
			@ApiResponse(responseCode = "500", description = "Internal server error", content = {
					@Content(mediaType = "application/problem+json", schema = @Schema(implementation = ErrorResponseType.class)) }) })
	@PostMapping(value = "/evote/qa/verify", consumes = { MediaType.APPLICATION_JSON_VALUE }, produces = {
			MediaType.APPLICATION_JSON_VALUE, MediaType.APPLICATION_PROBLEM_JSON_VALUE })
	public ResponseEntity<?> verify(@RequestBody VerifyAQRequestType data) {
		try {
			service.verify(data);
			return ResponseEntity.ok().contentType(MediaType.APPLICATION_JSON).body(new VerifyAQResponseType(true));
		} catch (EVoteAppException e) {
			logger.error(e.getMessage());
			return ResponseEntity.status(422).contentType(MediaType.APPLICATION_PROBLEM_JSON)
					.body(new ErrorResponseType(e.getErrorCode().name(), e.getErrorCode().getDescription()));
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			return ResponseEntity.internalServerError().contentType(MediaType.APPLICATION_PROBLEM_JSON)
					.body(new ErrorResponseType(e.getClass().getName(), e.getMessage()));
		} finally {
			logger.info("Verify completed");
		}
	}

	@Operation(summary = "Get counts of qualified attributes")
	@ApiResponses(value = { @ApiResponse(responseCode = "200", description = "Ok", content = {
			@Content(mediaType = MediaType.APPLICATION_JSON_VALUE, schema = @Schema(implementation = CountAQResultType.class)) }),
			@ApiResponse(responseCode = "500", description = "Internal server error", content = {
					@Content(mediaType = "application/problem+json", schema = @Schema(implementation = ErrorResponseType.class)) }) })
	@GetMapping("/evote/qa/count")
	public ResponseEntity<?> getCounts(
			@Parameter(description = "Data to filer qualified attributes", example = "2000-10-31T01:30:00.000-05:00") @RequestParam(name = "startDate", required = false) @DateTimeFormat(iso = DateTimeFormat.ISO.DATE_TIME) Date startDate) {
		try {
			CountAQResultType counts = service.getCounts(startDate);
			return ResponseEntity.ok(counts);
		} catch (EVoteAppException e) {
			logger.error(e.getMessage(), e);
			return ResponseEntity.status(422).contentType(MediaType.APPLICATION_PROBLEM_JSON)
					.body(new ErrorResponseType(e.getErrorCode().name(), e.getErrorCode().getDescription()));
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			return ResponseEntity.internalServerError().contentType(MediaType.APPLICATION_PROBLEM_JSON)
					.body(new ErrorResponseType(e.getClass().getName(), e.getMessage()));
		} finally {
			logger.info("Counts completed");
		}
	}

}
